from llm_bridge.logic.message_preprocess import document_processor
from llm_bridge.logic.message_preprocess.file_type_checker import get_file_type
from llm_bridge.type.message import Message, Role


async def preprocess_messages(messages: list[Message]) -> None:
    for message in messages:
        await extract_text_files_to_message(message)


async def extract_text_files_to_message(message: Message) -> None:
    indices_to_delete = []
    for i, file_url in enumerate(message.files[::-1]):
        file_type, sub_type = get_file_type(file_url)
        if file_type != "text":
            continue
        filename = file_url.rsplit('/', 1)[-1].split('-', 1)[1]
        file_text = await document_processor.extract_text_from_file(file_url)
        message.text = f"{filename}: \n{file_text}\n{message.text}"
        indices_to_delete.append(len(message.files) - 1 - i)

    for index in sorted(indices_to_delete, reverse=True):
        del message.files[index]


def extract_system_messages(messages: list[Message]) -> str:
    system_message = ""
    indices_to_delete = []
    for i, message in enumerate(messages):
        if message.role != Role.System:
            continue
        system_message += f"{message.text}\n"
        indices_to_delete.append(i)

    for index in sorted(indices_to_delete):
        del messages[index]

    return system_message



from typing import AsyncGenerator

from google import genai
from google.genai import types

from llm_bridge.client.chat_client import ChatClient
from llm_bridge.type.chat_response import ChatResponse
from llm_bridge.type.model_message.gemini_message import GeminiMessage


class GeminiClient(ChatClient):
    def __init__(
            self,
            model: str,
            messages: list[GeminiMessage],
            temperature: float,
            client: genai.Client,
            config: types.GenerateContentConfig,
    ):
        self.model = model
        self.messages = messages
        self.temperature = temperature
        self.client = client
        self.config = config

    async def generate_response(self) -> ChatResponse | AsyncGenerator[ChatResponse, None]:
        raise NotImplementedError



import logging
import re
from typing import AsyncGenerator

import httpx
from fastapi import HTTPException

from llm_bridge.client.model_client.claude_client import ClaudeClient
from llm_bridge.type.chat_response import ChatResponse
from llm_bridge.type.serializer import serialize


class StreamClaudeClient(ClaudeClient):
    async def generate_response(self) -> AsyncGenerator[ChatResponse, None]:
        try:
            logging.info(f"messages: {self.messages}")

            async def chunk_generator() -> AsyncGenerator[ChatResponse, None]:
                try:
                    async with self.client.messages.stream(
                        model=self.model,
                        max_tokens=4096,
                        temperature=self.temperature,
                        system=self.system,
                        messages=serialize(self.messages)
                    ) as stream:
                        async for response_delta in stream.text_stream:
                            yield ChatResponse(text=response_delta)
                except Exception as e:
                    logging.exception(e)
                    yield ChatResponse(error=str(e))

            return chunk_generator()

        except httpx.HTTPStatusError as e:
            status_code = e.response.status_code
            text = e.response.text
            raise HTTPException(status_code=status_code, detail=text)
        except Exception as e:
            logging.exception(e)
            match = re.search(r'\d{3}', str(e))
            if match:
                error_code = int(match.group(0))
            else:
                error_code = 500

            raise HTTPException(status_code=error_code, detail=str(e))



from openai.types.chat import ChatCompletionContentPartTextParam, ChatCompletionContentPartImageParam, \
    ChatCompletionContentPartInputAudioParam
from openai.types.chat.chat_completion_content_part_image_param import ImageURL
from openai.types.chat.chat_completion_content_part_input_audio_param import InputAudio

from llm_bridge.logic.chat_generate import media_processor
from llm_bridge.logic.message_preprocess.file_type_checker import get_file_type
from llm_bridge.type.model_message import gpt_message
from llm_bridge.type.model_message.gpt_message import GptMessage
from llm_bridge.type.message import Message


async def convert_message_to_gpt(message: Message) -> GptMessage:
    role = message.role
    text = message.text
    file_urls = message.files

    content = []

    if text:
        text_content = ChatCompletionContentPartTextParam(type="text", text=text)
        content.append(text_content)

    for file_url in file_urls:
        file_type, sub_type = get_file_type(file_url)
        if file_type == "image":
            image_url = await media_processor.get_gpt_image_content_from_url(file_url)
            image_content = ChatCompletionContentPartImageParam(
                type="image_url",
                image_url=ImageURL(url=image_url)
            )
            content.append(image_content)
        elif file_type == "audio":
            encoded_string = await media_processor.get_gpt_audio_content_from_url(file_url)
            audio_content = ChatCompletionContentPartInputAudioParam(
                type="input_audio",
                input_audio=InputAudio(data=encoded_string, format=sub_type)
            )
            content.append(audio_content)
        else:
            text_content = ChatCompletionContentPartTextParam(
                type="text",
                text=f"\n{file_url}: {file_type}/{sub_type} not supported by the current model.\n"
            )
            content.append(text_content)

    return GptMessage(role=role, content=content)



from dataclasses import dataclass

from openai.types.chat import ChatCompletionContentPartTextParam, ChatCompletionContentPartImageParam, \
    ChatCompletionContentPartInputAudioParam

from llm_bridge.type.message import Role


@dataclass
class GptMessage:
    role: Role
    content: list[
        ChatCompletionContentPartTextParam | ChatCompletionContentPartImageParam | ChatCompletionContentPartInputAudioParam
    ]



import mimetypes

from llm_bridge.logic.message_preprocess.code_file_extensions import code_file_extensions


def is_file_type_supported(file_name: str) -> bool:
    if file_name.endswith(('.doc', '.xls', '.ppt')):
        return False
    else:
        return True


def get_file_type(file_name: str) -> tuple[str, str]:
    if file_name.endswith(code_file_extensions):
        return 'text', 'code'
    if file_name.endswith('.pdf'):
        return 'text', 'pdf'
    if file_name.endswith(('.docx', '.doc')):
        return 'text', 'word'
    if file_name.endswith(('.xlsx', '.xls')):
        return 'text', 'excel'
    if file_name.endswith(('.pptx', '.ppt')):
        return 'text', 'ppt'
    if file_name.endswith('.mp3'):
        return 'audio', 'mp3'
    if file_name.endswith('.wav'):
        return 'audio', 'wav'
    if file_name.endswith('.webm'): # currently unable to tell audio / video
        return 'audio', 'webm'

    mime_type, _ = mimetypes.guess_type(file_name)
    if mime_type:
        return mime_type.split('/')[0], mime_type.split('/')[1]
    return 'unknown', 'unknown'



from google.genai import types

from llm_bridge.logic.chat_generate import media_processor
from llm_bridge.logic.message_preprocess.file_type_checker import get_file_type
from llm_bridge.type.model_message.gemini_message import GeminiMessage
from llm_bridge.type.message import Message, Role


async def convert_message_to_gemini(message: Message) -> GeminiMessage:
    role = message.role
    text = message.text
    file_urls = message.files

    if role == Role.System:
        role = Role.User
    if role == Role.Assistant:
        role = Role.Model

    parts = [types.Part.from_text(text=text)]

    for file_url in file_urls:
        file_type, sub_type = get_file_type(file_url)
        if file_type == "image":
            img_bytes, media_type = await media_processor.get_gemini_image_content_from_url(file_url)
            parts.append(types.Part.from_bytes(data=img_bytes, mime_type=media_type))
        elif file_type == "audio":
            audio_bytes, media_type = await media_processor.get_gemini_audio_content_from_url(file_url)
            parts.append(types.Part.from_bytes(data=audio_bytes, mime_type=media_type))
        else:
            text_content = types.Part.from_text(
                text=f"\n{file_url}: {file_type}/{sub_type} not supported by the current model.\n"
            )
            parts.append(text_content)


    return GeminiMessage(parts=parts, role=role)



from llm_bridge.logic.chat_generate import media_processor
from llm_bridge.logic.message_preprocess.file_type_checker import get_file_type
from llm_bridge.type.model_message import claude_message
from llm_bridge.type.model_message.claude_message import ClaudeMessage
from llm_bridge.type.message import Message, Role


async def convert_message_to_claude(message: Message) -> ClaudeMessage:
    role = message.role
    text = message.text
    file_urls = message.files

    if role == Role.System:
        role = Role.User

    content = []

    if text:
        text_content = claude_message.TextContent(type="text", text=text)
        content.append(text_content)

    for file_url in file_urls:
        file_type, sub_type = get_file_type(file_url)
        if file_type == "image":
            image_contents = await media_processor.get_claude_image_content_from_url(file_url)
            content.append(image_contents)
            continue
        text_content = claude_message.TextContent(
            type="text",
            text=f"\n{file_url}: {file_type}/{sub_type} not supported by the current model.\n"
        )
        content.append(text_content)

    return ClaudeMessage(role=role, content=content)



from dataclasses import dataclass
from enum import Enum


class Role(Enum):
    User = "user"
    Assistant = "assistant"
    System = "system"
    Model = "model"


@dataclass
class Message:
    role: Role
    text: str
    files: list[str]



import pytest
from openai import AsyncOpenAI

from llm_bridge.client.implementations.gpt.non_stream_gpt_client import NonStreamGPTClient
from llm_bridge.logic.chat_generate.chat_client_factory import create_chat_client
from llm_bridge.type.model_message.gpt_message import GptMessage, TextContent
from llm_bridge.type.message import Message, Role


@pytest.fixture
def sample_messages():
    return [
        Message(role=Role.System, text="You are a helpful assistant.", files=[]),
        Message(role=Role.User, text="Hello", files=[])
    ]


@pytest.mark.asyncio
async def test_create_gpt_client_openai():
    messages = [Message(role=Role.User, text="Hello", files=[])]
    model = "gpt-4o"
    api_type = "OpenAI"
    temperature = 0
    stream = False
    api_keys = {
        "OPENAI_API_KEY": "test-key"
    }

    client = await create_chat_client(
        messages=messages,
        model=model,
        api_type=api_type,
        temperature=temperature,
        stream=stream,
        api_keys=api_keys
    )

    assert isinstance(client, NonStreamGPTClient)
    assert client.model == model
    assert client.temperature == temperature
    assert client.api_type == api_type
    assert isinstance(client.client, AsyncOpenAI)

    assert len(client.messages) == len(messages)
    assert isinstance(client.messages[0], GptMessage)
    assert client.messages[0].role == Role.User
    assert isinstance(client.messages[0].content, list)
    assert isinstance(client.messages[0].content[0], TextContent)
    assert client.messages[0].content[0].text == "Hello"



from typing import AsyncGenerator

from llm_bridge.type.chat_response import ChatResponse


class ChatClient:
    async def generate_response(self) -> ChatResponse | AsyncGenerator[ChatResponse, None]:
        raise NotImplementedError
